@@ -46,7 +46,6 @@ CalculatorApplet::CalculatorApplet( QObject *parent, const QVariantList &args )
 {
     setAspectRatioMode(Plasma::IgnoreAspectRatio);
     resize(300,300);
-    setMinimumSize(50,50);
     setPopupIcon("accessories-calculator");
 }
 
@@ -54,6 +53,7 @@ QGraphicsWidget *CalculatorApplet::graphicsWidget()
 {
     if (!m_widget) {
         m_widget = new QGraphicsWidget(this);
+        m_widget->setMinimumSize(150,150);
         m_layout = new QGraphicsGridLayout(m_widget);
 
         previousAddSubOperation=calcNone;
@@ -183,7 +183,6 @@ QGraphicsWidget *CalculatorApplet::graphicsWidget()
         qreal left, top, right, bottom;
         getContentsMargins(&left, &top, &right, &bottom);
         m_widget->setMinimumSize(m_layout->sizeHint(Qt::MinimumSize) + QSizeF(left + right, top + bottom));
-        m_widget->resize(300,300);
 
         QAction *copy = new QAction(i18n( "Copy" ), this);
         actions.append(copy);
@@ -372,7 +371,12 @@ void CalculatorApplet::slotAddClicked()
 
     //kDebug() << "'+' was clixed, parsed to: " << currentValue << endl;
 
-    if (previousAddSubOperation!=calcNone) {
+    if (previousMulDivOperation != calcNone) {
+        calculate(currentValue, previousMulDivOperation);
+        previousMulDivOperation = calcNone;
+        calculate(factor, previousAddSubOperation);
+        factor = 0;
+    } else if (previousAddSubOperation != calcNone) {
         calculate(currentValue, previousAddSubOperation);
     } else {
         sum = currentValue;
@@ -387,7 +391,12 @@ void CalculatorApplet::slotSubtractClicked()
 {
     double currentValue = KGlobal::locale()->readNumber(inputText);
 
-    if (previousAddSubOperation!=calcNone) {
+    if (previousMulDivOperation != calcNone) {
+        calculate(currentValue, previousMulDivOperation);
+        previousMulDivOperation = calcNone;
+        calculate(factor, previousAddSubOperation);
+        factor = 0;
+    } else if (previousAddSubOperation != calcNone) {
         calculate(currentValue, previousAddSubOperation);
     } else {
         sum = currentValue;
