@@ -1,7 +1,7 @@
 /***************************************************************************
  *   Copyright (C) 2007 by Tobias Koenig <tokoe@kde.org>                   *
  *   Copyright (C) 2008 by Marco Martin <notmart@gmail.com>                *
- *   Copyright (C) 2008 Matthias Fuchs <mat69@gmx.net>                     *
+ *   Copyright (C) 2008-2010 Matthias Fuchs <mat69@gmx.net>                *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -23,14 +23,12 @@
 
 #include <QtCore/QPropertyAnimation>
 #include <QtCore/QTimer>
-#include <QtGui/QAction>
 #include <QtGui/QGraphicsLinearLayout>
 #include <QtGui/QGraphicsScene>
 #include <QtGui/QGraphicsView>
 #include <QtGui/QGraphicsSceneMouseEvent>
 #include <QtGui/QGraphicsSceneWheelEvent>
-#include <QtGui/QLabel>
-#include <QtGui/QVBoxLayout>
+#include <QtGui/QSortFilterProxyModel>
 
 #include <KConfigDialog>
 #include <KDatePicker>
@@ -58,6 +56,7 @@
 #include <plasma/tooltipmanager.h>
 
 #include "arrowwidget.h"
+#include "comicmodel.h"
 #include "configwidget.h"
 #include "fullviewwidget.h"
 #include "imagewidget.h"
@@ -107,6 +106,7 @@ class ChooseStripNumDialog : public KDialog
 
 ComicApplet::ComicApplet( QObject *parent, const QVariantList &args )
     : Plasma::PopupApplet( parent, args ),
+      mDifferentComic( true ),
       mShowPreviousButton( false ),
       mShowNextButton( false ),
       mShowComicUrl( false ),
@@ -142,6 +142,13 @@ void ComicApplet::init()
 
     loadConfig();
 
+    mEngine = dataEngine( "comic" );
+    mModel = new ComicModel( mEngine->query( "providers" ), mTabIdentifier, this );
+    mProxy = new QSortFilterProxyModel( this );
+    mProxy->setSourceModel( mModel );
+    mProxy->setSortCaseSensitivity( Qt::CaseInsensitive );
+    mProxy->sort( 1, Qt::AscendingOrder );
+
     mCurrentDay = QDate::currentDate();
     mDateChangedTimer = new QTimer( this );
     connect( mDateChangedTimer, SIGNAL( timeout() ), this, SLOT( checkDayChanged() ) );
@@ -192,7 +199,7 @@ void ComicApplet::init()
     mActions.append( mActionStorePosition );
     connect( mActionStorePosition, SIGNAL( triggered( bool ) ), this, SLOT( slotStorePosition() ) );
 
-    updateTabBar();
+    updateUsedComics();
     changeComic( true );
 }
 
@@ -296,27 +303,47 @@ void ComicApplet::dataUpdated( const QString&, const Plasma::DataEngine::Data &d
     setBusy( false );
     setConfigurationRequired( false );
     slotStartTimer();
+
+    //there was an error, display information as image
     if ( data[ "Error" ].toBool() ) {
-        if ( !data[ "Previous identifier suffix" ].toString().isEmpty() ) {
+        QPixmap errorPic( 500, 400 );
+        errorPic.fill();
+        QPainter p( &errorPic );
+        QFont font = Plasma::Theme::defaultTheme()->font( Plasma::Theme::DefaultFont );
+        font.setPointSize( 24 );
+        p.setPen( QColor( 0, 0, 0 ) );
+        p.setFont( font );
+        QString title = i18n( "Getting comic strip failed:" );
+        p.drawText( QRect( 10, 10 , 480, 100 ), Qt::TextWordWrap | Qt::AlignHCenter | Qt::AlignVCenter, title );
+        QString text = i18n( "Maybe there is no Internet connection.\nMaybe the comic plugin is broken.\nAnother reason might be that there is no comic for this day/number/string, so choosing a different one might work." );
+
+        mPreviousIdentifierSuffix = data[ "Previous identifier suffix" ].toString();
+        mNextIdentifierSuffix.clear();
+        if ( !mPreviousIdentifierSuffix.isEmpty() ) {
             if ( !data[ "Identifier" ].toString().isEmpty() ) {
                 mIdentifierError = data[ "Identifier" ].toString();
             }
-            updateComic( data[ "Previous identifier suffix" ].toString() );
-        } else {
-            setConfigurationRequired( true );
+            text.append( i18n( "\n\nChoose the previous strip to go to the last cached strip." ) );
         }
-        return;
+
+        font.setPointSize( 16 );
+        p.setFont( font );
+        p.drawText( QRect( 10, 120 , 480, 270 ), Qt::TextWordWrap | Qt::AlignLeft, text );
+
+        mImage = errorPic.toImage();
+        mAdditionalText = title + text;
+    } else {
+        mImage = data[ "Image" ].value<QImage>();
+        mPreviousIdentifierSuffix = data[ "Previous identifier suffix" ].toString();
+        mNextIdentifierSuffix = data[ "Next identifier suffix" ].toString();
+        mAdditionalText = data[ "Additional text" ].toString();
     }
 
-    mImage = data[ "Image" ].value<QImage>();
     mWebsiteUrl = data[ "Website Url" ].value<KUrl>();
     setAssociatedApplicationUrls(mWebsiteUrl);
     mShopUrl = data[ "Shop Url" ].value<KUrl>();
-    mNextIdentifierSuffix = data[ "Next identifier suffix" ].toString();
-    mPreviousIdentifierSuffix = data[ "Previous identifier suffix" ].toString();
     mFirstIdentifierSuffix = data[ "First strip identifier suffix" ].toString();
     mStripTitle = data[ "Strip title" ].toString();
-    mAdditionalText = data[ "Additional text" ].toString();
     mComicAuthor = data[ "Comic Author" ].toString();
     mComicTitle = data[ "Title" ].toString();
     mSuffixType = data[ "SuffixType" ].toString();
@@ -341,7 +368,7 @@ void ComicApplet::dataUpdated( const QString&, const Plasma::DataEngine::Data &d
     KConfigGroup cg = config();
     mShownIdentifierSuffix = "";
     if ( mSuffixType == "Number" ) {
-        mShownIdentifierSuffix = i18nc("an abbrevation for Number", "# %1", mCurrentIdentifierSuffix);
+        mShownIdentifierSuffix = i18nc("an abbreviation for Number", "# %1", mCurrentIdentifierSuffix);
         int tempNum = mCurrentIdentifierSuffix.toInt();
         if ( mMaxStripNum[ mComicIdentifier ] < tempNum ) {
             mMaxStripNum[ mComicIdentifier ] = tempNum;
@@ -380,6 +407,15 @@ void ComicApplet::dataUpdated( const QString&, const Plasma::DataEngine::Data &d
     if ( !mImage.isNull() ) {
         QTimer::singleShot( 1, this, SLOT( updateSize()) );//HACK
     }
+
+    //prefetch the previous and following comic for faster navigation
+    if ( !mNextIdentifierSuffix.isEmpty() ) {
+        mEngine->query( mComicIdentifier + ':' + mNextIdentifierSuffix );
+    }
+
+    if ( !mPreviousIdentifierSuffix.isEmpty() ) {
+        mEngine->query( mComicIdentifier + ':' + mPreviousIdentifierSuffix );
+    }
 }
 
 void ComicApplet::createConfigurationInterface( KConfigDialog *parent )
@@ -387,8 +423,7 @@ void ComicApplet::createConfigurationInterface( KConfigDialog *parent )
     //to not have tab switches while configurating things
     mReloadTimer->stop();
 
-    mConfigWidget = new ConfigWidget( dataEngine( "comic" ), parent );
-    mConfigWidget->setComicIdentifier( mTabIdentifier );
+    mConfigWidget = new ConfigWidget( dataEngine( "comic" ), mModel, mTabIdentifier, mProxy, parent );
     mConfigWidget->setShowComicUrl( mShowComicUrl );
     mConfigWidget->setShowComicAuthor( mShowComicAuthor );
     mConfigWidget->setShowComicTitle( mShowComicTitle );
@@ -400,6 +435,7 @@ void ComicApplet::createConfigurationInterface( KConfigDialog *parent )
     mConfigWidget->setHideTabBar( !mShowTabBar );
     mConfigWidget->setUseTabs( mUseTabs );
     mConfigWidget->setSwitchTabs( mSwitchTabs );
+    mConfigWidget->setTabView( mTabView - 1);//-1 because counting starts at 0, yet we use flags that start at 1
 
     parent->setButtons( KDialog::Ok | KDialog::Cancel | KDialog::Apply );
     parent->addPage( mConfigWidget->comicSettings, i18n( "General" ), icon(), i18n( "Press the \"Get New Comics ...\" button to install comics." ) );
@@ -413,12 +449,6 @@ void ComicApplet::createConfigurationInterface( KConfigDialog *parent )
 
 void ComicApplet::applyConfig()
 {
-    const bool differentComic = ( mComicIdentifier != mConfigWidget->comicIdentifier().at( 0 ) );
-
-    mTabIdentifier = mConfigWidget->comicIdentifier();
-    mComicIdentifier = mTabIdentifier.at( 0 );
-    mTabText = mConfigWidget->comicName();
-    mComicTitle = mTabText.at( 0 );
     mShowComicUrl = mConfigWidget->showComicUrl();
     mShowComicAuthor = mConfigWidget->showComicAuthor();
     mShowComicTitle = mConfigWidget->showComicTitle();
@@ -430,13 +460,14 @@ void ComicApplet::applyConfig()
     mShowTabBar = !mConfigWidget->hideTabBar();
     mUseTabs = mConfigWidget->useTabs();
     mSwitchTabs = mConfigWidget->switchTabs();
+    mTabView = mConfigWidget->tabView() + 1;//+1 because counting starts at 0, yet we use flags that start at 1
 
+    updateUsedComics();
     slotStartTimer();
-    updateTabBar();
     saveConfig();
     buttonBar();
 
-    changeComic( differentComic );
+    changeComic( mDifferentComic );
 }
 
 void ComicApplet::changeComic( bool differentComic )
@@ -457,19 +488,45 @@ void ComicApplet::changeComic( bool differentComic )
     }
 }
 
-void ComicApplet::updateTabBar()
+void ComicApplet::updateUsedComics()
 {
-    for ( int i = mTabBar->count(); i > 0; --i ) {
-        mTabBar->removeTab( i - 1 );
-    }
-    for ( int i = 0; i < mTabIdentifier.count(); ++i ) {
-        mTabBar->addTab( mTabText.at( i ) );
+    const QString oldIdentifier = mComicIdentifier;
+    mTabBar->removeAllTabs();
+    mTabIdentifier.clear();
+    mComicIdentifier.clear();
+    mComicTitle.clear();
+
+    bool isFirst = true;
+    QModelIndex data;
+    for ( int i = 0; i < mProxy->rowCount(); ++i ) {
+        if ( mProxy->index( i, 0 ).data( Qt::CheckStateRole) == Qt::Checked ) {
+            data = mProxy->index( i, 1 );
+
+            if ( isFirst ) {
+                isFirst = false;
+                mComicIdentifier = data.data( Qt::UserRole ).toString();
+                mDifferentComic = ( oldIdentifier != mComicIdentifier );
+                mComicTitle = data.data().toString();
+            }
+
+            QIcon icon;
+            QString name;
+            if ( mTabView & ShowText ) {
+                name = data.data().toString();
+            }
+            if ( mTabView & ShowIcon ) {
+                icon = data.data( Qt::DecorationRole ).value<QIcon>();
+            }
+
+            mTabBar->addTab( icon, name );
+            mTabIdentifier << data.data( Qt::UserRole ).toString();
+        }
     }
 }
 
 void ComicApplet::slotTabChanged( int newIndex )
 {
-    if (newIndex >= mTabIdentifier.count()) {
+    if ( newIndex >= mTabIdentifier.count() ) {
         return;
     }
 
@@ -508,10 +565,9 @@ void ComicApplet::loadConfig()
 
     mSwitchTabTime = cg.readEntry( "switchTabTime", 10 );// 10 seconds as default
     mShowTabBar = cg.readEntry( "showTabBar", true );
-    mTabText = cg.readEntry( "tabText", QStringList( QString() ) );
-    mComicTitle = mTabText.count() ? mTabText.at( 0 ) : QString();
     mUseTabs = cg.readEntry( "useTabs", false );
     mSwitchTabs = cg.readEntry( "switchTabs", false );
+    mTabView = cg.readEntry( "tabView", ShowText | ShowIcon );
     mSavingDir = cg.readEntry( "savingDir", QString() );
     mOldSource = mComicIdentifier + ':' + mStoredIdentifierSuffix;
 
@@ -531,9 +587,9 @@ void ComicApplet::saveConfig()
     cg.writeEntry( "switchTabTime", mSwitchTabTime );
     cg.writeEntry( "showTabBar", mShowTabBar );
     cg.writeEntry( "tabIdentifier", mTabIdentifier );
-    cg.writeEntry( "tabText", mTabText );
     cg.writeEntry( "useTabs", mUseTabs );
     cg.writeEntry( "switchTabs", mSwitchTabs );
+    cg.writeEntry( "tabView", mTabView );
     cg.writeEntry( "savingDir", mSavingDir );
 }
 
@@ -731,10 +787,8 @@ void ComicApplet::updateComic( const QString &identifierSuffix )
         mEngine->connectSource( identifier, this );
         const Plasma::DataEngine::Data data = mEngine->query( identifier );
 
-        if ( data[ "Error" ].toBool() && data[ "Previous identifier suffix" ].toString().isEmpty() ) {
-            setBusy( false );
-            setConfigurationRequired( true );
-            slotStartTimer();
+        if ( data[ "Error" ].toBool() ) {
+            dataUpdated( QString(), data );
         }
     } else {
         kError() << "Either no identifier was specified or the engine could not be created:" << "id" << mComicIdentifier << "engine valid:" << ( mEngine && mEngine->isValid() );
@@ -990,7 +1044,7 @@ void ComicApplet::buttonBar()
 void ComicApplet::fullView()
 {
     if ( !mFullViewWidget ) {
-        mFullViewWidget = new FullViewWidget();
+        mFullViewWidget = new FullViewWidget( view() );
     }
 
     if ( !mFullViewWidget->isVisible() ) {
