@@ -1,7 +1,7 @@
 /***************************************************************************
  *   Copyright (C) 2007 by Tobias Koenig <tokoe@kde.org>                   *
  *   Copyright (C) 2008 by Marco Martin <notmart@gmail.com>                *
- *   Copyright (C) 2008 Matthias Fuchs <mat69@gmx.net>                     *
+ *   Copyright (C) 2008-2010 Matthias Fuchs <mat69@gmx.net>                *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -26,25 +26,93 @@
 #include <QtGui/QImage>
 
 #include <KUrl>
-#include <plasma/applet.h>
-#include <plasma/dataengine.h>
+#include <Plasma/DataEngine>
+#include <Plasma/Label>
+#include <Plasma/PopupApplet>
+#include <Plasma/TabBar>
 
 class ArrowWidget;
+class ComicModel;
 class ConfigWidget;
-class FadingItem;
 class FullViewWidget;
 class ImageWidget;
-class QTimer;
 class QAction;
+class QGraphicsLayout;
+class QPropertyAnimation;
+class QSortFilterProxyModel;
+class QTimer;
 
 namespace Plasma {
 class Frame;
-class Label;
 class PushButton;
-class TabBar;
 }
 
-class ComicApplet : public Plasma::Applet
+//Helper class, sets the sizeHint to 0 if the TabBar is hidden
+class ComicTabBar : public Plasma::TabBar
+{
+    public:
+        ComicTabBar( QGraphicsWidget *parent = 0 ) : TabBar( parent ) {}
+        ~ComicTabBar() {}
+
+        void removeAllTabs()
+        {
+            while ( this->count() ) {
+                this->removeTab( 0 );
+            }
+        }
+
+    protected:
+        QSizeF sizeHint( Qt::SizeHint which, const QSizeF &constraint = QSizeF() ) const
+        {
+            if ( !isVisible() ) {
+                return QSizeF( 0, 0 );
+            }
+            return QGraphicsWidget::sizeHint( which, constraint );
+        }
+
+        void hideEvent( QHideEvent *event )
+        {
+            updateGeometry();
+            QGraphicsWidget::hideEvent( event );
+        }
+
+        void showEvent( QShowEvent *event )
+        {
+            updateGeometry();
+            QGraphicsWidget::showEvent( event );
+        }
+};
+
+//Helper class, sets the sizeHint to 0 if the Label is hidden
+class ComicLabel : public Plasma::Label
+{
+    public:
+        ComicLabel( QGraphicsWidget *parent = 0 ) : Plasma::Label( parent ) {}
+        ~ComicLabel() {}
+
+    protected:
+        QSizeF sizeHint( Qt::SizeHint which, const QSizeF &constraint = QSizeF() ) const
+        {
+            if ( !isVisible() ) {
+                return QSizeF( 0, 0 );
+            }
+            return QGraphicsProxyWidget::sizeHint( which, constraint );
+        }
+
+        void hideEvent( QHideEvent *event )
+        {
+            updateGeometry();
+            QGraphicsProxyWidget::hideEvent( event );
+        }
+
+        void showEvent( QShowEvent *event )
+        {
+            updateGeometry();
+            QGraphicsProxyWidget::showEvent( event );
+        }
+};
+
+class ComicApplet : public Plasma::PopupApplet
 {
     Q_OBJECT
 
@@ -55,6 +123,8 @@ class ComicApplet : public Plasma::Applet
         void init();
         virtual QList<QAction*> contextualActions();
 
+        QGraphicsWidget *graphicsWidget();
+
     public Q_SLOTS:
         void dataUpdated( const QString &name, const Plasma::DataEngine::Data &data );
         void createConfigurationInterface( KConfigDialog *parent );
@@ -78,26 +148,26 @@ class ComicApplet : public Plasma::Applet
         void applyConfig();
         void checkDayChanged();
         void buttonBar();
-        void createLayout();
+        void fullView();
+        void updateSize();
 
     protected:
-        void mousePressEvent( QGraphicsSceneMouseEvent* );
-        void mouseReleaseEvent( QGraphicsSceneMouseEvent* );
-        void wheelEvent( QGraphicsSceneWheelEvent* );
-        void hoverEnterEvent( QGraphicsSceneHoverEvent* );
-        void hoverLeaveEvent( QGraphicsSceneHoverEvent* );
-        void constraintsEvent( Plasma::Constraints constraints );
         QSizeF sizeHint(Qt::SizeHint which, const QSizeF &constraint = QSizeF()) const;
+        bool eventFilter( QObject *receiver, QEvent *event );
 
     private:
         void changeComic( bool differentComic );
         void updateComic( const QString &identifierSuffix = QString() );
-        void updateTabBar();
+        void updateUsedComics();
         void updateButtons();
         void updateContextMenu();
         void loadConfig();
         void saveConfig();
-        void updateSize();
+        bool isInPanel() const;
+
+    private:
+        ComicModel *mModel;
+        QSortFilterProxyModel *mProxy;
 
         QImage mImage;
         QDate mCurrentDay;
@@ -116,7 +186,10 @@ class ComicApplet : public Plasma::Applet
         QString mCurrentIdentifierSuffix;
         QString mStoredIdentifierSuffix;
         QString mIdentifierError;
+        QString mSavingDir;
+        QString mOldSource;
         ConfigWidget *mConfigWidget;
+        bool mDifferentComic;
         bool mScaleComic;
         bool mShowPreviousButton;
         bool mShowNextButton;
@@ -129,6 +202,7 @@ class ComicApplet : public Plasma::Applet
         QTimer *mDateChangedTimer;
         QTimer *mReloadTimer;
         QList<QAction*> mActions;
+        QGraphicsWidget *mMainWidget;
         FullViewWidget *mFullViewWidget;
         QAction *mActionGoFirst;
         QAction *mActionGoLast;
@@ -143,12 +217,13 @@ class ComicApplet : public Plasma::Applet
         QSizeF mIdealSize;
         Plasma::DataEngine *mEngine;
         Plasma::Frame *mFrame;
-        FadingItem *mFadingItem;
-        Plasma::Label *mLabelId;
-        Plasma::Label *mLabelTop;
-        Plasma::Label *mLabelUrl;
+        QPropertyAnimation *mFrameAnim;
+        ComicLabel *mLabelId;
+        ComicLabel *mLabelTop;
+        ComicLabel *mLabelUrl;
         Plasma::PushButton *mPrevButton;
         Plasma::PushButton *mNextButton;
+        Plasma::PushButton *mZoomButton;
 
         ImageWidget *mImageWidget;
         ArrowWidget *mLeftArrow;
@@ -159,9 +234,14 @@ class ComicApplet : public Plasma::Applet
         bool mShowTabBar;
         bool mSwitchTabs;
         int mSwitchTabTime;
-        Plasma::TabBar *mTabBar;
+        ComicTabBar *mTabBar;
         QStringList mTabIdentifier;
-        QStringList mTabText;
+
+        enum TabView {
+            ShowText = 0x1,
+            ShowIcon = 0x2
+        };
+        int mTabView;
 };
 
 K_EXPORT_PLASMA_APPLET( comic, ComicApplet )
