@@ -316,11 +316,15 @@ void NowPlaying::playerAdded(const QString &name)
         kDebug() << "Installing" << name << "as watched player";
         m_watchingPlayer = name;
 
+        delete m_controller;
         m_controller = dataEngine("nowplaying")->serviceForSource(m_watchingPlayer);
-        m_controller->associateWidget(m_positionSlider, "seek");
-        m_controller->associateWidget(m_volumeSlider, "volume");
-        emit controllerChanged(m_controller);
+        m_controller->setParent(this);
+        if (m_controller) {
+            m_controller->associateWidget(m_positionSlider, "seek");
+            m_controller->associateWidget(m_volumeSlider, "volume");
+        }
 
+        emit controllerChanged(m_controller);
         dataEngine("nowplaying")->connectSource(m_watchingPlayer, this, 500);
     }
 }
@@ -350,11 +354,15 @@ void NowPlaying::findPlayer()
     } else {
         m_watchingPlayer = players.first();
 
+        delete m_controller;
         m_controller = dataEngine("nowplaying")->serviceForSource(m_watchingPlayer);
-        m_controller->associateWidget(m_positionSlider, "seek");
-        m_controller->associateWidget(m_volumeSlider, "volume");
-        emit controllerChanged(m_controller);
+        m_controller->setParent(this);
+        if (m_controller) {
+            m_controller->associateWidget(m_positionSlider, "seek");
+            m_controller->associateWidget(m_volumeSlider, "volume");
+        }
 
+        emit controllerChanged(m_controller);
         kDebug() << "Installing" << m_watchingPlayer << "as watched player";
         dataEngine("nowplaying")->connectSource(m_watchingPlayer, this, 999);
     }
