@@ -20,61 +20,70 @@
 
 #include "pastebin.h"
 
-#include <QApplication>
-#include <QClipboard>
-#include <QDrag>
-#include <QGraphicsLinearLayout>
-#include <QGraphicsScene>
-#include <QGraphicsSceneDragDropEvent>
-#include <QMimeData>
-#include <QFile>
-#include <QBuffer>
-#include <QPainter>
-#include <QPaintEngine>
-#include <QSignalMapper>
-
-#include <KAction>
-#include <KDebug>
-#include <KLocale>
 #include <KConfigDialog>
 #include <KStandardAction>
 #include <KToolInvocation>
 #include <KNotification>
+#include <KAction>
 
 #include <kmimetype.h>
-#include <ktemporaryfile.h>
+#include <KTemporaryFile>
+
 
 #include <kio/global.h>
 #include <kio/job.h>
 
 #include <Plasma/Theme>
+#include <Plasma/Service>
+#include <Plasma/ServiceJob>
+
+#include <QApplication>
+#include <QClipboard>
+#include <QGraphicsLinearLayout>
+#include <QGraphicsScene>
+#include <QGraphicsSceneDragDropEvent>
+#include <QFile>
+#include <QBuffer>
+#include <QPainter>
+#include <QPaintEngine>
+#include <QSignalMapper>
+#include <QPropertyAnimation>
+
 
 Pastebin::Pastebin(QObject *parent, const QVariantList &args)
-    : Plasma::Applet(parent, args), m_textServer(0),
-    m_imageServer(0), m_textBackend(0), m_imageBackend(0),
-    m_historySize(3), m_signalMapper(new QSignalMapper()), m_paste(0),
-    m_topSeparator(0), m_bottomSeparator(0)
+    : Plasma::Applet(parent, args),
+      m_textBackend(0), m_imageBackend(0), m_imagePrivacy(0),
+      m_historySize(3), m_signalMapper(new QSignalMapper()), m_paste(0),
+      m_topSeparator(0), m_bottomSeparator(0), m_servers(0)
 {
     setAcceptDrops(true);
     setHasConfigurationInterface(true);
     setAspectRatioMode(Plasma::ConstrainedSquare);
     setMinimumSize(16, 16);
     resize(150, 150);
-    timer = new QTimer(this);
-    connect(timer, SIGNAL(timeout()), this, SLOT(showErrors()));
+    m_timer = new QTimer(this);
+    m_hoverText = i18n("Drop!");
+    connect(m_timer, SIGNAL(timeout()), this, SLOT(showErrors()));
 
     connect(m_signalMapper, SIGNAL(mapped(const QString &)),
              this, SLOT(copyToClipboard(const QString &)));
+    connect(this, SIGNAL(activate()), this, SLOT(postClipboard()));
+
+    Plasma::DataEngine *engine = dataEngine("pastebin");
+    m_postingService = engine->serviceForSource("");
+    connect(m_postingService, SIGNAL(finished(Plasma::ServiceJob*)), this, SLOT(postingFinished(Plasma::ServiceJob*)));
+    engine->connectSource("textservers", this);
+    engine->connectSource("imageservers", this);
 }
 
 Pastebin::~Pastebin()
 {
     delete m_topSeparator;
     delete m_bottomSeparator;
-    delete m_icon;
+    delete m_postingService;
     QString history;
     for (int i = 0; i < m_actionHistory.size(); ++i) {
-        history.prepend(m_actionHistory.at(i)->text());
+        history.prepend(m_actionHistory.at(i)->toolTip());
         history.prepend("|");
         delete m_actionHistory.at(i);
     }
@@ -83,81 +92,24 @@ Pastebin::~Pastebin()
     cg.writeEntry("History", history);
 }
 
-void Pastebin::setImageServer(int backend)
-{
-    delete m_imageServer;
-
-    switch(backend) {
-
-    case Pastebin::IMAGEBINCA:
-        m_imageServer = static_cast<ImagebinCAServer*>(new ImagebinCAServer(config()));
-        break;
-
-    case Pastebin::IMAGESHACK:
-        m_imageServer = static_cast<ImageshackServer*>(new ImageshackServer(config()));
-        break;
-    }
-
-    m_imageBackend = backend;
-    connect(m_imageServer, SIGNAL(postFinished(QString)),
-            this, SLOT(showResults(QString)));
-    connect(m_imageServer, SIGNAL(postError()),
-            this, SLOT(showErrors()));
-}
-
-void Pastebin::setTextServer(int backend)
-{
-    delete m_textServer;
-
-    switch(backend) {
-
-    case Pastebin::PASTEBINCA:
-        m_textServer = static_cast<PastebinCAServer*>(new PastebinCAServer(config()));
-        break;
-
-    case Pastebin::PASTEBINCOM:
-        m_textServer = static_cast<PastebinCOMServer*>(new PastebinCOMServer(config()));
-        break;
-    }
-
-    m_textBackend = backend;
-    connect(m_textServer, SIGNAL(postFinished(QString)),
-            this, SLOT(showResults(QString)));
-    connect(m_textServer, SIGNAL(postError()),
-            this, SLOT(showErrors()));
-}
-
-void Pastebin::setHistorySize(int max)
-{
-    if (max <= 0)
-        max = 0;
-
-    while (max < m_actionHistory.count()) {
-        delete m_actionHistory.takeFirst();
-    }
-
-    m_historySize = max;
-}
-
 void Pastebin::init()
 {
     KConfigGroup cg = config();
-    int textBackend = cg.readEntry("TextBackend", "0").toInt();
-    int imageBackend = cg.readEntry("ImageBackend", "0").toInt();
+
+    m_textBackend = cg.readEntry("TextBackend", "0").toInt();
+    m_imageBackend = cg.readEntry("ImageBackend", "0").toInt();
+    m_imagePrivacy = cg.readEntry("imagePrivacy", "0").toInt();
     int historySize = cg.readEntry("HistorySize", "3").toInt();
-    QStringList history = cg.readEntry("History", "").split("|", QString::SkipEmptyParts);
+
+    QStringList history = cg.readEntry("History", "").split('|', QString::SkipEmptyParts);
 
     for (int i = 0; i < history.size(); ++i) {
         addToHistory(history.at(i));
     }
 
-    setTextServer(textBackend);
-    setImageServer(imageBackend);
     setHistorySize(historySize);
-
     setActionState(Idle);
     setInteractionState(Waiting);
-    m_icon = new KIcon("edit-paste"); // TODO: make member (for caching)
 
     updateTheme();
     connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), SLOT(updateTheme()));
@@ -165,6 +117,23 @@ void Pastebin::init()
     Plasma::ToolTipManager::self()->setContent(this, toolTipData);
 }
 
+void Pastebin::dataUpdated(const QString &sourceName, const Plasma::DataEngine::Data &data)
+{
+    //TODO: use the mappings!
+}
+
+void Pastebin::setHistorySize(int max)
+{
+    if (max <= 0) {
+        max = 0;
+    }
+    while (max < m_actionHistory.count()) {
+        delete m_actionHistory.takeFirst();
+    }
+
+    m_historySize = max;
+}
+
 void Pastebin::updateTheme()
 {
     m_font = Plasma::Theme::defaultTheme()->font(Plasma::Theme::DefaultFont);
@@ -225,7 +194,7 @@ void Pastebin::setActionState(ActionState state)
             toolTipData.setImage(KIcon("dialog-cancel"));
             // Notification ...
             QTimer::singleShot(15000, this, SLOT(resetActionState()));
-            timer->stop();
+            m_timer->stop();
             break;
         case IdleSuccess:
             setBusy(false);
@@ -233,7 +202,7 @@ void Pastebin::setActionState(ActionState state)
             toolTipData.setImage(KIcon("dialog-ok"));
             // Notification ...
             QTimer::singleShot(15000, this, SLOT(resetActionState()));
-            timer->stop();
+            m_timer->stop();
             break;
         case Sending:
             setBusy(true);
@@ -252,10 +221,16 @@ void Pastebin::setActionState(ActionState state)
 void Pastebin::constraintsEvent(Plasma::Constraints constraints)
 {
     if (constraints & (Plasma::FormFactorConstraint | Plasma::SizeConstraint)) {
-        int minSize = KGlobalSettings::smallestReadableFont().pointSize();
-        int dynSize = qMax((qreal)1.0, (qreal)(qMin(contentsRect().width(), contentsRect().height()) / 4));
-        //kDebug() << "Min : Dyn" << minSize << dynSize << qMax(minSize, dynSize);
-        m_font.setPointSize(qMax(minSize, dynSize));
+        QFontMetrics fm(m_font);
+        int textWidthInPixels = fm.width(m_hoverText);
+        m_font.setPointSize((int)(m_font.pointSize() * ((double)contentsRect().width() / textWidthInPixels)));
+    }
+    if (constraints & Plasma::FormFactorConstraint) {
+        if (formFactor() == Plasma::Horizontal || formFactor() == Plasma::Vertical ) {
+            setPreferredSize(-1, -1);
+        } else {
+            setPreferredSize(150, 150);
+        }
     }
 }
 
@@ -340,11 +315,12 @@ void Pastebin::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *, con
         QPixmap iconok = KIcon("dialog-cancel").pixmap(QSize(iconsize, iconsize));
         paintPixmap(p, iconok, iconRect, pix_alpha);
     } else {
-        QPixmap iconpix = m_icon->pixmap(QSize(iconsize, iconsize));
-        paintPixmap(p, iconpix, iconRect, pix_alpha);
+        QPixmap iconpix = KIcon("edit-paste").pixmap(QSize(iconsize, iconsize));
+        if (!iconpix.isNull()) {
+            paintPixmap(p, iconpix, iconRect, pix_alpha);
+        }
     }
 
-
     // Draw background
     if (m_interactionState == DraggedOver) {
         m_fgColor.setAlphaF(m_alpha);
@@ -365,13 +341,13 @@ void Pastebin::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *, con
     qreal round_radius = 35.0;
     p->drawRoundedRect(contentsRect, round_radius / proportion, round_radius, Qt::RelativeSize);
 
-    QString hoverText = i18n("Drop!");
     if (m_interactionState == DraggedOver) {
         m_fgColor.setAlphaF(m_alpha * 0.8);
     } else if (m_interactionState == Hovered) {
         m_fgColor.setAlphaF(m_alpha * 0.6);
     }
-    p->drawText(contentsRect, Qt::AlignCenter, hoverText);
+
+    p->drawText(contentsRect, Qt::AlignCenter, m_hoverText);
     p->setPen(m_fgColor);
 
     if (m_actionState == IdleSuccess) {
@@ -387,7 +363,7 @@ void Pastebin::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *, con
         p->drawText(contentsRect, Qt::AlignCenter, i18n("Error: Try Again"));
     } else {
         p->setFont(m_font);
-        p->drawText(contentsRect, Qt::AlignCenter, hoverText);
+        p->drawText(contentsRect, Qt::AlignCenter, m_hoverText);
     }
 }
 
@@ -409,28 +385,36 @@ void Pastebin::showOverlay(bool show)
         return;
     }
     m_fadeIn = show;
-    const int FadeInDuration = 400;
 
-    if (m_animId != -1) {
-        Plasma::Animator::self()->stopCustomAnimation(m_animId);
+    QPropertyAnimation *animation = m_animation.data();
+    if (!animation) {
+        animation = new QPropertyAnimation(this, "animationUpdate");
+        animation->setDuration(400);
+        animation->setStartValue(0.0);
+        animation->setEndValue(1.0);
+        animation->setEasingCurve(QEasingCurve::Linear);
+        m_animation = animation;
+    } else if (animation->state() == QAbstractAnimation::Running) {
+        animation->pause();
     }
-    m_animId = Plasma::Animator::self()->customAnimation(40 / (1000 / FadeInDuration), FadeInDuration,
-                                                      Plasma::Animator::EaseOutCurve, this,
-                                                      "animationUpdate");
+
+    if (m_fadeIn) {
+        animation->setDirection(QAbstractAnimation::Forward);
+        animation->start(QAbstractAnimation::KeepWhenStopped);
+    } else {
+        animation->setDirection(QAbstractAnimation::Backward);
+        animation->start(QAbstractAnimation::DeleteWhenStopped);
+    }
+}
+
+qreal Pastebin::animationValue() const
+{
+    return m_alpha;
 }
 
 void Pastebin::animationUpdate(qreal progress)
 {
-    if (progress == 1) {
-        m_animId = -1;
-    }
-    if (!m_fadeIn) {
-        qreal new_alpha = m_fadeIn ? progress : 1 - progress;
-        m_alpha = qMin(new_alpha, m_alpha);
-    } else {
-        m_alpha = m_fadeIn ? progress : 1 - progress;
-    }
-    m_alpha = qMax(qreal(0.0), m_alpha);
+    m_alpha = progress;
     update();
 }
 
@@ -441,60 +425,156 @@ void Pastebin::createConfigurationInterface(KConfigDialog *parent)
 
     QWidget *general = new QWidget();
     uiConfig.setupUi(general);
+
     connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
+    connect(parent, SIGNAL(cancelClicked()), this, SLOT(closeServerDialog()));
     parent->addPage(general, i18n("General"), Applet::icon());
 
     uiConfig.textServer->setCurrentIndex(m_textBackend);
     uiConfig.imageServer->setCurrentIndex(m_imageBackend);
     uiConfig.historySize->setValue(m_historySize);
+    uiConfig.imagePrivacy->setCurrentIndex(m_imagePrivacy);
 
-    QWidget *servers = new QWidget();
-    uiServers.setupUi(servers);
-    parent->addPage(servers, i18n("Servers"), Applet::icon());
+    uiConfig.editTextServer->setIcon(KIcon("document-edit"));
+    uiConfig.editImageServer->setIcon(KIcon("document-edit"));
 
-    QString pastebincaURL = cg.readEntry("pastebinca", "http://pastebin.ca");
-    uiServers.pastebinca->setText(pastebincaURL);
+    connect(uiConfig.editTextServer, SIGNAL(clicked()), this, SLOT(editTextServer()));
+    connect(uiConfig.editImageServer, SIGNAL(clicked()), this, SLOT(editImageServer()));
+}
 
-    QString pastebincomURL = cg.readEntry("pastebincom", "http://pastebin.com");
-    uiServers.pastebincom->setText(pastebincomURL);
+void Pastebin::editTextServer()
+{
+    KConfigGroup cg = config();
 
-    QString imagebincaURL = cg.readEntry("imagebinca", "http://imagebin.ca");
-    uiServers.imagebinca->setText(imagebincaURL);
+    // avoid memory leaks as its not modal and the user can click again
+    // on the edit button
+    delete m_servers;
 
-    QString imageshackURL = cg.readEntry("imageshack", "http://imageshack.us");
-    uiServers.imageshack->setText(imageshackURL);
+    m_servers = new QWidget();
+    uiServers.setupUi(m_servers);
+    m_servers->show();
+    connect(uiServers.buttonBox, SIGNAL(accepted()), this, SLOT(saveTextServer()));
+    connect(uiServers.buttonBox, SIGNAL(rejected()), this, SLOT(closeServerDialog()));
+    QString serverAddress;
+    switch(uiConfig.textServer->currentIndex()) {
+        case Pastebin::PASTEBINCA:
+            serverAddress = cg.readEntry("pastebinca", "http://pastebin.ca");
+            break;
+
+        case Pastebin::PASTEBINCOM:
+           serverAddress = cg.readEntry("pastebincom", "http://pastebin.com");
+           break;
+    }
+
+    uiServers.server->setText(serverAddress);
+}
+
+void Pastebin::saveTextServer()
+{
+    KConfigGroup cg = config();
+
+    switch(uiConfig.textServer->currentIndex()) {
+        case Pastebin::PASTEBINCA:
+            cg.writeEntry("pastebinca", uiServers.server->text());
+            break;
+
+        case Pastebin::PASTEBINCOM:
+            cg.writeEntry("pastebincom", uiServers.server->text());
+            break;
+    }
+
+    closeServerDialog();
+}
+
+void Pastebin::closeServerDialog()
+{
+    delete m_servers;
+    m_servers = 0;
+}
+
+void Pastebin::editImageServer()
+{
+    KConfigGroup cg = config();
+
+    // avoid memory leaks as its not modal and the user can click again
+    // on the edit button
+    delete m_servers;
+
+    m_servers = new QWidget();
+    uiServers.setupUi(m_servers);
+    m_servers->show();
+    connect(uiServers.buttonBox, SIGNAL(accepted()), this, SLOT(saveImageServer()));
+    connect( uiServers.buttonBox, SIGNAL( rejected() ), this, SLOT( closeServerDialog() ) );
+    QString serverAddress;
+    switch(uiConfig.imageServer->currentIndex()) {
+        case Pastebin::IMAGEBINCA:
+            serverAddress = cg.readEntry("imagebinca", "http://imagebin.ca");
+            break;
+
+        case Pastebin::IMAGESHACK:
+            serverAddress = cg.readEntry("imageshack", "http://imageshack.us");
+            break;
+
+        case Pastebin::SIMPLESTIMAGEHOSTING:
+            serverAddress = cg.readEntry("simplestimagehosting",
+                                         "http://api.simplest-image-hosting.net");
+            break;
+
+        case Pastebin::IMGUR:
+            serverAddress = cg.readEntry("imgur", "http://imgur.com/api/upload");
+            break;
+    }
+
+    uiServers.server->setText(serverAddress);
+}
+
+void Pastebin::saveImageServer()
+{
+    KConfigGroup cg = config();
+
+    switch(uiConfig.imageServer->currentIndex()) {
+        case Pastebin::IMAGEBINCA:
+            cg.writeEntry("imagebinca", uiServers.server->text());
+            break;
+
+        case Pastebin::IMAGESHACK:
+            cg.writeEntry("imageshack", uiServers.server->text());
+            break;
+
+        case Pastebin::SIMPLESTIMAGEHOSTING:
+            cg.writeEntry("simplestimagehosting", uiServers.server->text());
+            break;
+
+        case Pastebin::IMGUR:
+            cg.writeEntry("imgur", uiServers.server->text());
+            break;
+
+    }
+    closeServerDialog();
 }
 
 void Pastebin::configAccepted()
 {
     KConfigGroup cg = config();
-    int textBackend = uiConfig.textServer->currentIndex();
-    int imageBackend = uiConfig.imageServer->currentIndex();
+    m_textBackend = uiConfig.textServer->currentIndex();
+    m_imageBackend = uiConfig.imageServer->currentIndex();
+    m_imagePrivacy = uiConfig.imagePrivacy->currentIndex();
+
     int historySize = uiConfig.historySize->value();
+    setHistorySize(historySize);
 
-    QString pastebincaURL = uiServers.pastebinca->text();
-    QString pastebincomURL = uiServers.pastebincom->text();
-    QString imagebincaURL = uiServers.imagebinca->text();
-    QString imageshackURL = uiServers.imageshack->text();
-
-    cg.writeEntry("TextBackend", textBackend);
-    cg.writeEntry("ImageBackend", imageBackend);
+    cg.writeEntry("TextBackend", m_textBackend);
+    cg.writeEntry("ImageBackend", m_imageBackend);
+    cg.writeEntry("imagePrivacy", m_imagePrivacy);
     cg.writeEntry("HistorySize", historySize);
 
-    cg.writeEntry("pastebinca", pastebincaURL);
-    cg.writeEntry("pastebincom", pastebincomURL);
-    cg.writeEntry("imagebinca", imagebincaURL);
-    cg.writeEntry("imageshack", imageshackURL);
-
-    setTextServer(textBackend);
-    setImageServer(imageBackend);
-    setHistorySize(historySize);
+    closeServerDialog();
     emit configNeedsSaving();
 }
 
 void Pastebin::showResults(const QString &url)
 {
-    timer->stop();
+    m_timer->stop();
     m_url = url;
     setActionState(IdleSuccess);
     copyToClipboard(url);
@@ -504,8 +584,9 @@ void Pastebin::showResults(const QString &url)
 
 void Pastebin::copyToClipboard(const QString &url)
 {
-    QApplication::clipboard()->setText(url);
+    QApplication::clipboard()->setText(url, lastMode);
     kDebug() << "Copying:" << url;
+    m_oldUrl = url;
     QPixmap pix = KIcon("edit-paste").pixmap(KIconLoader::SizeMedium, KIconLoader::SizeMedium);
 
     KNotification *notify = new KNotification("urlcopied");
@@ -523,9 +604,13 @@ void Pastebin::showErrors()
     setActionState(IdleError);
 }
 
-void Pastebin::openLink()
+void Pastebin::openLink(bool old)
 {
-    KToolInvocation::invokeBrowser(m_url);
+    if(old) {
+      KToolInvocation::invokeBrowser(m_oldUrl);
+    } else {
+      KToolInvocation::invokeBrowser(m_url);
+    }
 }
 
 void Pastebin::resetActionState()
@@ -539,27 +624,12 @@ void Pastebin::mousePressEvent(QGraphicsSceneMouseEvent *event)
     if (m_url.isEmpty() || event->button() != Qt::LeftButton) {
         Plasma::Applet::mousePressEvent(event);
     } else {
-        openLink();
+        openLink(false);
     }
     if (event->button() == Qt::MidButton) {
         if (m_actionState == Idle) {
             // paste clipboard content
-#ifdef Q_WS_WIN
-            // Same as for D'n'D, Windows doesn't pass any actual image data when pasting
-            // image files. Though, it does provide us with those files' Urls. Since posting
-            // multiple images isn't yet implemented - we'll use first Url from list
-            QImage image;
-            QString imageFileName;
-            if (QApplication::clipboard()->mimeData()->hasUrls()) {
-                imageFileName = QApplication::clipboard()->mimeData()->urls().at(0).toLocalFile();
-                image.load(imageFileName);
-                postContent(imageFileName, image);
-            } else {
-                postContent(QApplication::clipboard()->mimeData()->text(), image);
-            };
-#else
-            postContent(QApplication::clipboard()->text(), QApplication::clipboard()->image());
-#endif //Q_WS_WIN
+            postClipboard(true);
         } else {
             // Now releasing the middlebutton click copies to clipboard
             event->accept();
@@ -605,6 +675,7 @@ void Pastebin::dragMoveEvent(QGraphicsSceneDragDropEvent *event)
 void Pastebin::dropEvent(QGraphicsSceneDragDropEvent *event)
 {
     if (event->mimeData()->objectName() != QString("Pastebin-applet")) {
+        lastMode = QClipboard::Clipboard;
 #ifdef Q_WS_WIN
         // Apparently, Windows doesn't pass any actual image data when drag'n'dropping
         // image files. Though, it does provide us with those files' Urls. Since posting
@@ -661,7 +732,6 @@ QList<QAction*> Pastebin::contextualActions()
 
     m_contextualActions.append(m_paste);
     m_contextualActions.append(m_topSeparator);
-
     m_contextualActions.append(m_actionHistory);
 
     if (!m_actionHistory.isEmpty()) {
@@ -673,6 +743,12 @@ QList<QAction*> Pastebin::contextualActions()
 
 void Pastebin::postClipboard()
 {
+    postClipboard(false);
+}
+
+void Pastebin::postClipboard(bool preferSelection)
+{
+    lastMode = QClipboard::Clipboard;
 #ifdef Q_WS_WIN
 // Same as for D'n'D, Windows doesn't pass any actual image data when pasting
 // image files. Though, it does provide us with those files' Urls. Since posting
@@ -687,74 +763,118 @@ void Pastebin::postClipboard()
         postContent(QApplication::clipboard()->mimeData()->text(), image);
     };
 #else
-    postContent(QApplication::clipboard()->text(), QApplication::clipboard()->image());
+    if (preferSelection) {
+        lastMode = QApplication::clipboard()->supportsSelection() ? QClipboard::Selection : QClipboard::Clipboard;
+        postContent(QApplication::clipboard()->text(lastMode), QApplication::clipboard()->image(lastMode));
+    } else {
+        postContent(QApplication::clipboard()->text(), QApplication::clipboard()->image());
+    }
 #endif //Q_WS_WIN
 }
 
 void Pastebin::postContent(QString text, QImage imageData)
 {
-    bool image = false;
-    bool validPath = false;
-
-    setActionState(Sending);
-    timer->start(20000);
-
     KUrl testPath(text);
-    validPath = QFile::exists(testPath.toLocalFile());
+    bool validPath = QFile::exists(testPath.toLocalFile());
+    bool image = false;
+    QString operationName;
 
     if (validPath) {
         KMimeType::Ptr type = KMimeType::findByPath(testPath.path());
 
         if (type->name().indexOf("image/") != -1) {
+            operationName = "image";
             image = true;
+        } else {
+            operationName = "text";
         }
+    } else if (imageData.isNull()) {
+        operationName = "literalText";
     } else {
-        if (!imageData.isNull()) {
-            image = true;
+        operationName = "image";
+        image = true;
+
+        KTemporaryFile tempFile;
+        if (tempFile.open()) {
+            tempFile.setAutoRemove(false);
+
+            QDataStream stream(&tempFile);
+            QByteArray data;
+            QBuffer buffer(&data);
+
+            buffer.open(QIODevice::ReadWrite);
+            imageData.save(&buffer, "PNG");
+            stream.writeRawData(data, data.size());
+            tempFile.close();
+
+            text = tempFile.fileName();
+        } else {
+            setActionState(IdleError);
+            return;
         }
     }
 
-    if (!image) {
-        if (validPath) {
-            QFile file(testPath.toLocalFile());
-            file.open(QIODevice::ReadOnly);
-            QTextStream in(&file);
-            text = in.readAll();
+    KConfigGroup cg = config();
+
+    KConfigGroup ops = m_postingService->operationDescription(operationName);
+    kDebug() << image << validPath << operationName;
+    if (image) {
+        switch(m_imageBackend) {
+            case Pastebin::IMAGEBINCA:
+                ops.writeEntry("server", cg.readEntry("imagebinca", "http://imagebin.ca"));
+                break;
+
+            case Pastebin::IMAGESHACK:
+                ops.writeEntry("server", cg.readEntry("imageshack", "http://imageshack.us"));
+                break;
+
+            case Pastebin::SIMPLESTIMAGEHOSTING:
+                ops.writeEntry("server", cg.readEntry("simplestimagehosting",
+                                                      "http://api.simplest-image-hosting.net"));
+                break;
+
+            case Pastebin::IMGUR:
+                ops.writeEntry("server", cg.readEntry("imgur", "http://imgur.com/api/upload"));
+                break;
         }
 
-        // upload text
-        m_textServer->post(text);
+        ops.writeEntry("fileName", text);
+        ops.writeEntry("privacy", m_imagePrivacy);
+        ops.writeEntry("backend", m_imageBackend);
     } else {
-        //upload image
-        if (validPath) {
-            m_imageServer->post(testPath.toLocalFile());
-        } else {
-            KTemporaryFile tempFile;
-            if (tempFile.open()) {
-                tempFile.setAutoRemove(false);
+        switch (m_textBackend) {
+            case Pastebin::PASTEBINCA:
+                ops.writeEntry("server", cg.readEntry("pastebinca", "http://pastebin.ca"));
+                break;
 
-                QDataStream stream(&tempFile);
-
-                QByteArray data;
-                QBuffer buffer(&data);
-                buffer.open(QIODevice::ReadWrite);
-
-                QImage image = imageData;
-                image.save(&buffer, "JPEG");
-                stream.writeRawData(data, data.size());
-
-                KUrl t(tempFile.fileName());
-                tempFile.close();
-
-                m_imageServer->post(t.path());
-
-            } else {
-                setActionState(IdleError);
-                timer->stop();
-            }
+            case Pastebin::PASTEBINCOM:
+                ops.writeEntry("server", cg.readEntry("pastebincom", "http://pastebin.com"));
+                break;
         }
+
+        if (validPath) {
+            ops.writeEntry("fileName", text);
+        } else {
+            ops.writeEntry("text", text);
+        }
+
+        ops.writeEntry("backend", m_textBackend);
+    }
+
+    Plasma::ServiceJob *job = m_postingService->startOperationCall(ops);
+    setActionState(Sending);
+    m_timer->start(20000);
+}
+
+void Pastebin::postingFinished(Plasma::ServiceJob *job)
+{
+    if (job->error()) {
+        showErrors();
+    } else {
+        showResults(job->result().toString());
     }
 }
 
 
+
 #include "pastebin.moc"
