@@ -22,11 +22,13 @@
 #include <KIcon>
 #include <KDebug>
 #include <KToolInvocation>
-#include <conversion/converter.h>
-#include <conversion/unitcategory.h>
+#include <KUnitConversion/Converter>
+#include <KUnitConversion/UnitCategory>
 
 #define CONVERSION_CHAR '>'
 
+using namespace KUnitConversion;
+
 class StringParser
 {
 public:
@@ -124,10 +126,18 @@ ConverterRunner::ConverterRunner(QObject* parent, const QVariantList &args)
     Q_UNUSED(args)
     setObjectName("Converter");
 
+    m_separators << QString(CONVERSION_CHAR);
+    m_separators << i18nc("list of words that can used as amount of 'unit1' [in|to|as] 'unit2'",
+                          "in;to;as").split(';');
+
     //can not ignore commands: we have things like m4
     setIgnoredTypes(Plasma::RunnerContext::Directory | Plasma::RunnerContext::File |
                     Plasma::RunnerContext::NetworkLocation);
 
+    QString description = i18n("Converts the value of :q: when :q: is made up of "
+                               "\"value unit [>, to, as, in] unit\". You can use the "
+                               "Unit converter applet to find all available units.");
+    addSyntax(Plasma::RunnerSyntax("=:q:", description));
 }
 
 ConverterRunner::~ConverterRunner()
@@ -158,54 +168,84 @@ void ConverterRunner::match(Plasma::RunnerContext &context)
         }
     }
 
-    QStringList separators;
-    separators << QString(CONVERSION_CHAR) << i18nc("amount of 'unit1' in 'unit2'", "in")
-               << i18nc("amount of 'unit1' as 'unit2'", "as");
     const QString s = cmd.get(StringParser::GetString);
 
-    if (!s.isEmpty() && !separators.contains(s)) {
+    if (!s.isEmpty() && !m_separators.contains(s)) {
         unit1 += ' ' + s;
     }
-    if (s.isEmpty() || !separators.contains(s)) {
-        cmd.pass(separators);
+    if (s.isEmpty() || !m_separators.contains(s)) {
+        cmd.pass(m_separators);
     }
     unit2 = cmd.rest();
 
-    Conversion::UnitCategory* category = Conversion::Converter::self()->categoryForUnit(unit1);
-    if (!category)
-        return;
-    Conversion::Value v = category->convert(Conversion::Value(value, unit1), unit2);
-    if (v.isValid()) {
+    Converter converter;
+    UnitCategory* category = converter.categoryForUnit(unit1);
+    bool found = false;
+    if (category->id() == InvalidCategory) {
+        foreach (category, converter.categories()) {
+            foreach (const QString& s, category->allUnits()) {
+                if (s.compare(unit1, Qt::CaseInsensitive) == 0) {
+                    found = true;
+                    break;
+                }
+            }
+            if (found) {
+                break;
+            }
+        }
+        if (!found) {
+            return;
+        }
+    }
+
+    QList<UnitPtr> units;
+
+    if (!unit2.isEmpty()) {
+        UnitPtr u = category->unit(unit2);
+        if (!u.isNull() && u->isValid()) {
+            units.append(u);
+            config().writeEntry(category->name(), u->symbol());
+        } else {
+            const QStringList unitStrings = category->allUnits();
+            QSet<UnitPtr> matchingUnits;
+            foreach (const QString& s, unitStrings) {
+                if (s.startsWith(unit2, Qt::CaseInsensitive)) {
+                    matchingUnits << category->unit(s);
+                }
+            }
+            units = matchingUnits.toList();
+            if (units.count() == 1) {
+                config().writeEntry(category->name(), units[0]->symbol());
+            }
+        }
+    } else {
+        units = category->mostCommonUnits();
+        UnitPtr u = category->unit(config().readEntry(category->name()));
+        if (!u.isNull() && units.indexOf(u) < 0) {
+            units << u;
+        }
+    }
+
+    UnitPtr u1 = category->unit(unit1);
+    foreach (const UnitPtr& u, units) {
+        if (u1 == u) {
+            continue;
+        }
+        Value v = category->convert(Value(value.toDouble(), u1), u);
         Plasma::QueryMatch match(this);
         match.setType(Plasma::QueryMatch::InformationalMatch);
         match.setIcon(KIcon("edit-copy"));
-        match.setText(v.toString());
+        match.setText(QString("%1 (%2)").arg(v.toString()).arg(u->symbol()));
         match.setData(v.number());
         context.addMatch(term, match);
-    } else if (!unit2.isEmpty()) {
-        const QStringList units = category->allUnits();
-        QSet<Conversion::Unit*> matchingUnits;
-        foreach (const QString& s, units) {
-            if (s.startsWith(unit2)) {
-                matchingUnits << category->unit(s);
-            }
-        }
-        foreach (const Conversion::Unit* u, matchingUnits) {
-            v = category->convert(Conversion::Value(value, unit1), u->symbol());
-            Plasma::QueryMatch match(this);
-            match.setType(Plasma::QueryMatch::InformationalMatch);
-            match.setIcon(KIcon("edit-copy"));
-            match.setText(v.toString());
-            match.setData(v.number());
-            context.addMatch(term, match);
-        }
     }
-    if (v.isValid() && !v.unit()->category()->description().isEmpty()) {
+
+    if (units.count() > 0 && !category->description().isEmpty()) {
         Plasma::QueryMatch match(this);
         match.setType(Plasma::QueryMatch::PossibleMatch);
         match.setIcon(KIcon("document-open-remote"));
-        match.setText(v.unit()->category()->description());
-        match.setData(v.unit()->category()->url().prettyUrl());
+        match.setText(category->description());
+        match.setData(category->url().prettyUrl());
         context.addMatch(term, match);
     }
 }
